'''
#####################
#  Rummy.py Server  #
#####################
'''
from card import *
from talk import *

import select
import socket
from random import shuffle
from time import sleep

# DEFAULT SETTINGS

BUFFER_SIZE = 100
TICK = 1				# Sleep 1 second between checking select

#

class Player:
	def __init__(self, name):
		self.name = name
		self.socket = socket
		self.hand = []
		self.turn_flag = False

class RummyServer:
	def __init__ (self, max_players, port):
		# Network settings
		self.max_players = max_players
		self.port = port
		
		# Game settings
		self.players = {} 		# addr:port => player instance
		self.player_order = []	# ID => addr:port 
		self.current = None		# ID of current player
		
		'''
		# Initalize piles
		self.pickup_pile = []
		for i in range(52):
			self.pickup_pile.append(Card(i%4, i%13))
		shuffle(self.pickup_pile)
		self.discard_pile = [self.pickup_pile.pop()]
		#
		'''
		
		self.srvsock = socket.socket( socket.AF_INET, socket.SOCK_STREAM ) 
		self.srvsock.setsockopt( socket.SOL_SOCKET, socket.SO_REUSEADDR, 1 ) 
		self.srvsock.bind( ("", port) ) 
		self.srvsock.listen( max_players )
		
		self.descriptors = [self.srvsock]
	
		print 'Rummy.py Server initialized on port %s.' % port

	# Current player info helper functions
	def get_current(self):
		if type(self.current) == int:
			return self.players[self.player_order[self.current]]
		return False

	def get_current_name(self):
		if type(self.current) == int:
			return self.players[self.player_order[self.current]].name
		return False
	
	def get_current_key(self):
		if type(self.current) == int:
			return self.player_order[self.current]
		return False
	#
	
	def run (self):
		
		print 'Server now running on port %s.' % (self.port)
		
		while True: 
			# Await an event on a readable socket descriptor 
			(sread, swrite, sexc) = select.select( self.descriptors, [], [], TICK )
			
			for sock in sread: 
			
				if sock == self.srvsock: 
					# New connection
					self.accept_new_connection()
					
				elif len(self.descriptors) > 2:
					# Received something on a client socket 
					host, port = sock.getpeername()
					request = sock.recv(BUFFER_SIZE)
					key = "%s:%s" % (host, port)
					is_valid_player = self.players.has_key(key)
					
					if request and is_valid_player:
						
						if request == ASK_GO:
							self.handle_go(sock, key)
						elif request == ACT_END:
							self.handle_end(sock, key)
						else:
							print "Unexpected request from %s: %s..." % (key, request)
							
					elif is_valid_player:
						# Client has left...
						self.clean_disconnection(sock, key)
						
					else:
						sock.send(MSG_FAIL) # This seems impossible
	
	def accept_new_connection (self):
		
		# Accept connection and add to known sockets
		newsock, (host, port) = self.srvsock.accept()
		self.descriptors.append( newsock )
		
		# Establish some info about player
		name = newsock.recv(BUFFER_SIZE)
		key = "%s:%s" % (host, port)
		
		# Add to players lists
		self.players[key] = Player(name)
		self.player_order.append(key)		# List of player keys,
		
		# Can we begin?
		if len(self.descriptors) == 3:
			self.current = 0
			
		newsock.send(MSG_OK)
		
		print "%s (%s) connected." % (name, key)
		
	def clean_disconnection(self, sock, key):
		sock.close()
		self.descriptors.remove(sock)
		
		print "%s (%s) disconnected." % (self.players[key].name, key)
		
		'''
		cards = self.players[key].hand # + self.players[key]...
		
		
		shuffle(cards)
		self.pickup_pile.extend(cards)
		'''
		
		del self.players[key]
		id = self.player_order.index(key)
		del self.player_order[id]
		
		if self.current == id:
			self.current += 1
			self.current %= len(self.players)
		
		
	# Request handler helper functions
	def handle_go(self, sock, key):
		print "%s (%s) asks to go..." % (self.players[key].name, key)
		if key == self.get_current_key():
			sock.send(MSG_OK)
		else:
			sock.send(MSG_FAIL)
	
	def handle_end(self, sock, key):
		if key == self.get_current_key():
			sock.send(MSG_OK)
			print "%s ends turn..." % self.get_current_name()
			self.current += 1
			self.current %= len(self.players)
			print "%s begins turn..." % self.get_current_name()
		else:
			sock.send(MSG_FAIL)

		
	#